#include <qfile.h>
#include <qstring.h>
#include <qevent.h>
#include "gfcurve.h"
#include "gfscale.h"
#include "gfcommon.h"
#include "gfprofile.h"
#include "gfcomments.h"
#include "gfplotdefs.h"
#include "gfdraw.h"

#ifndef GFFILEH
#define GFFILEH

/*

	GfFile class 
	


*/


class GfFile
{

public:
	GfFile(); 
	GfFile(char *filename);
	
	GfFile(const GfFile &); // copy constructor
	GfFile(const GfFile *); // copy constructor
	
	virtual ~GfFile();
	
	int read(char *filename, int, int,int append=GF_NO); // filename, xcol, ycol
	int read(QFile *,int,int,int append=GF_NO);
	
	int checkNumCols(char *filename);
	int checkNumCols(QFile *);
	
	int write(char *filename);
	int write(QFile*);
	
	int draw(GfDraw *);
	
	int setCurve(int,const GfCurve &);
	int appendCurve(const GfCurve &);
	int appendCurve(const GfCurve *);
	int removeCurve(int);
	int breakCurve(int curve,int point);
	
	int appendPoint(int c,int iv,float x, float y);
	int appendRow(int c,int iv,float *row);
	
	int setCurvePlotColumns(int,int,int);
    
	int setScale(const GfScale &);
	
	int setProfile(const GfProfile &);
	
	int setComments(const GfComments &);
	
	int setPlotDefs(const GfPlotDefs &);
	
	int numPoints() const ;
	
	int numCurves() const ;
	
	GfScale scale() const ;
	
	const GfCurve &curve(int ) const ;
	
	int curveLineType(int) const ;
	int curveLineWidth(int) const ;
	int curveColor(int) const ;
	float curvePatternLength(int) const ;
	
	int setCurveLineType(int, int);
	int setCurveLineWidth(int, int);
	int setCurveColor(int, int);
	int setCurvePatternLength(int, float);	
	
	GfProfile profile() const ;
	
	GfComments comments() const ;
	
	GfPlotDefs plotDefs() const ;
	
	void autoScale();
	
	QString fileName() const ;
	
	QString inFileComments() const ;
	void setInFileComments(const char *);
	
	void clear(); // removes all data
	
	int closestPoint(int,int, GfDraw *,int *curve,int *point) const ;
	
	GfCurve curveSegment(int curve,int start,int end) const ;	
	void drawSelection(GfDraw *gfd,int c,int start,int end,int color_step = 1);
	int getMaxMin(int c,float *xmax,float *xmin,float *ymax,float *ymin) const ;
	
	void test(); // fills in with testing plot
	void test_mc(); // fills a curve with 3 columns
	
	void print(OSTREAM *);
	
	void operator=(const GfFile &); // assignment operator
	

private:

	GfCurve **curves;
	
	GfScale xyscale;
	
	int numcurves;
	
	GfProfile gfprofile;
	
	GfComments gfcomments;
	
	GfPlotDefs gfplotdefs;
	
	QString file_name;
	
	QString in_file_comments;

	void init();
	
	void storage(int );
	
	int chk_line_size(char *);
	
	int auto_np(char *);
	
	int parse_data(char *line,int *i1, float *x, float *y,int,int);
	int parse_data_fixed(char *line,int *i1, float *x, float *y,int xcol,int ycol,int vlen);
	void strip_crlf(char *text);
	
	int read_extended(QFile *in,char *line,int append);
			
	GfCurve empty; // for curve() to return on bad request
	
};

OSTREAM &operator<<(OSTREAM &os, GfFile *m);
OSTREAM &operator<<(OSTREAM &os, GfFile &m);


#endif // GfFile
