#include "gfplotdefs.h"
#include "gfutil.h"



GfPlotDefs::GfPlotDefs()
{
#ifdef DEBUG1
	cout << "construct gfplotdefs\n";
#endif	

	init();
	
}

void GfPlotDefs::init()
{

	encap = GF_NO;  /* default to non encapsulated post */
	rotp = GF_YES;   /* default to rotated plot output */
	arrow_size = 0.7;   /* size of invec 11&12 arrows, 0 = scale with vector */
	symbol_size = 1.0 ; /* scale factor for symbols */
	psjoin = PSJOIN_ROUND; /* 0 = miter 1=round 2=bevel joins in ps output */
	font_size = PS_DEF_FONTSIZE; // font_size

	plot_axes = GF_YES; 
	plot_comments = GF_YES; 
	plot_curves = GF_YES; 

	x_plot_size = 5.;      /* plot sizes in inches */
	y_plot_size = 4.;
	
	plot_name = NULL;
	
	axes_font = PS_DEF_FONT;
	axes_color = BLACK;
	

}

GfPlotDefs::GfPlotDefs(const GfPlotDefs &source) // copy constructor
{
	init();

	encap=source.encapsulated();
	rotp = source.rotatePlot();
	arrow_size = source.arrowSize();
	symbol_size = source.symbolSize();
	psjoin = source.psJoin();
	font_size = source.fontSize();
	plot_axes = source.axesVisible();
	plot_comments = source.commentsVisible();
	plot_curves = source.curvesVisible();
	x_plot_size = source.xPlotSize();
	y_plot_size = source.yPlotSize();
	
	if(source.plotNameLen() > 0)
	{
		plot_name = (char *)malloc(source.plotNameLen()+1);
		if(plot_name == NULL)
		{
			cout << "out of memory in plotdefs\n";
			exit(-1);
		}
		source.plotName(plot_name);
	}
	axes_font = source.font();
	axes_color = source.color();
	

}

GfPlotDefs::GfPlotDefs(const GfPlotDefs *source) // copy constructor
{
	init();
	
	encap = source->encapsulated();
	rotp = source->rotatePlot();
	arrow_size = source->arrowSize();
	symbol_size = source->symbolSize();
	psjoin = source->psJoin();
	font_size = source->fontSize();
	plot_axes = source->axesVisible();
	plot_comments = source->commentsVisible();
	plot_curves = source->curvesVisible();
	x_plot_size = source->xPlotSize();
	y_plot_size = source->yPlotSize();
	
	if(source->plotNameLen() > 0)
	{
		plot_name = (char *)malloc(source->plotNameLen()+1);
		if(plot_name == NULL)
		{
			cout << "out of memory in plotdefs\n";
			exit(-1);
		}
		source->plotName(plot_name);
	}
	axes_font = source->font();
	axes_color = source->color();
	

}
GfPlotDefs::~GfPlotDefs()
{

	free(plot_name);

}


void GfPlotDefs::print(OSTREAM *os)
{
	*os << PLOTDEFSMAGIC <<"\n";
	*os << "C\tencapsulated: "<<encap<<"\n";
	*os << "C\trotateplot: "<<rotp<<"\n";
	*os << "C\tarrowsize: "<<arrow_size<<"\n";
	*os << "C\tsymbolsize: "<<symbol_size<<"\n";
	*os << "C\tpsjoin: "<<psjoin<<"\n";
	*os << "C\tfontsize: "<<font_size<<"\n";
	*os << "C\tplotaxes: "<<plot_axes<<"\n";
	*os << "C\tplotcomments: "<<plot_comments<<"\n";
	*os << "C\tplotcurves: "<<plot_curves<<"\n";
	*os << "C\txplotsize: "<<x_plot_size<<"\n";
	*os << "C\typlotsize: "<<y_plot_size<<"\n";
	*os << "C\taxesfont: "<<axes_font<<"\n";
	*os << "C\taxescolor: "<<axes_color<<"\n";

}

int GfPlotDefs::readFormat(QFile *in,char *line)
{

	if(strstr(line,PLOTDEFSMAGIC) == NULL)
		return(GF_BAD);

	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("encapsulated",line,&encap) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("rotateplot",line,&rotp) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getfloatval("arrowsize",line,&arrow_size) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getfloatval("symbolsize",line,&symbol_size) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("psjoin",line,&psjoin) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getfloatval("fontsize",line,&font_size) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("plotaxes",line,&plot_axes) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("plotcomments",line,&plot_comments) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("plotcurves",line,&plot_curves) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getfloatval("xplotsize",line,&x_plot_size) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getfloatval("yplotsize",line,&y_plot_size) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("axesfont",line,&axes_font) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("axescolor",line,&axes_color) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	return(GF_OK);
}

OSTREAM &operator<<(OSTREAM &os, GfPlotDefs *m)
{
	m->print(&os);
	return(os);
}
OSTREAM &operator<<(OSTREAM &os, GfPlotDefs &m)
{
	m.print(&os);
	return(os);
}

void GfPlotDefs::operator=(const GfPlotDefs &source)
{
	encap=source.encapsulated();
	rotp = source.rotatePlot();
	arrow_size = source.arrowSize();
	symbol_size = source.symbolSize();
	psjoin = source.psJoin();
	font_size = source.fontSize();
	plot_axes = source.axesVisible();
	plot_comments = source.commentsVisible();
	plot_curves = source.curvesVisible();
	x_plot_size = source.xPlotSize();
	y_plot_size = source.yPlotSize();
	
	if(source.plotNameLen() > 0)
	{
		if(plot_name != NULL) free(plot_name);
		
		plot_name = (char *)malloc(source.plotNameLen()+1);
		if(plot_name == NULL)
		{
			cout << "out of memory in plotdefs\n";
			exit(-1);
		}
		source.plotName(plot_name);
	}
	axes_font = source.font();
	axes_color = source.color();	
}

int GfPlotDefs::setRotatePlot(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
			rotp = t;
			return(GF_OK);
		break;
	
		default:
			return(GF_BAD);
		break;
	}			
}

int GfPlotDefs::setEncapsulated(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
			encap = t;
			return(GF_OK);
		break;
	
		default:
			return(GF_BAD);
		break;
	}
}


int GfPlotDefs::setPlotName(const char *s)
{
int len;
	len = strlen(s);
	
	if(len == 0)
	{
		free(plot_name);
		plot_name = NULL;
		return(GF_OK);
	}
	
	plot_name = (char *)realloc(plot_name,len+1);
	if(plot_name == NULL)
	{
		printf("outof memory in gfplotdefs\n");
		exit(-1);
	}
	
	strcpy(plot_name,s);
	
	return(GF_OK);

}


int GfPlotDefs::setPSJoin(int t)
{
	switch(t)
	{
		case PSJOIN_MITER:
		case PSJOIN_ROUND:
		case PSJOIN_BEVEL:
			psjoin = t;
			return(GF_OK);
		break;
		
		default:
			return(GF_BAD);
		break;
	}

}


int GfPlotDefs::setFontSize(float t)
{
	if(t > 0) 
	{
		font_size = t;
		return(GF_OK);
	}
	
	return(GF_BAD);
	
}


int GfPlotDefs::setSymbolSize(float t)
{
	if(t > 0) 
	{
		symbol_size = t;
		return(GF_OK);
	}
	
	return(GF_BAD);
}


int GfPlotDefs::setArrowSize(float t)
{
	if(t >= 0) 
	{
		arrow_size = t;
		return(GF_OK);
	}
	
	return(GF_BAD);

}

int GfPlotDefs::setPlotSize(float xs, float ys)
{
	if(xs > 0 && ys > 0) 
	{
		x_plot_size = xs;
		y_plot_size = ys;
		return(GF_OK);
	}
	
	return(GF_BAD);

}

	
int GfPlotDefs::setCurvesVisibility(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
			plot_curves = t;
			return(GF_OK);
		break;
	
		default:
			return(GF_BAD);
		break;
	}
}

int GfPlotDefs::setCommentsVisibility(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
			plot_comments = t;
			return(GF_OK);
		break;
	
		default:
			return(GF_BAD);
		break;
	}
}

int GfPlotDefs::setAxesVisibility(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
			plot_axes = t;
			return(GF_OK);
		break;
	
		default:
			return(GF_BAD);
		break;
	}
}

int GfPlotDefs::setFont(int t)
{
	if( t>= 0 && t < NUM_PSFONTS)
	{
		axes_font = t;
		return(GF_OK);
	}
	else
		return(GF_BAD);

}

int GfPlotDefs::setColor(int t)
{
	if( t>= 0 && t < NUM_PSCOLORS)
	{
		axes_color = t;
		return(GF_OK);
	}
	else
		return(GF_BAD);

}

int GfPlotDefs::rotatePlot() const 
{
	return(rotp);
}

int GfPlotDefs::encapsulated() const 
{
	return(encap);
}

void GfPlotDefs::plotName(char *s) const 
{
	if(plot_name != NULL)
		strcpy(s,plot_name);
	else
		strcpy(s,"");
}

int GfPlotDefs::plotNameLen() const 
{
	if(plot_name != NULL)
		return(strlen(plot_name));
	else
		return(0);
}

int GfPlotDefs::psJoin() const 
{
	return(psjoin);
}

float GfPlotDefs::fontSize() const 
{
	return(font_size);
}

float GfPlotDefs::symbolSize() const 
{
	return(symbol_size);
}

float GfPlotDefs::arrowSize() const 
{
	return(arrow_size);
}

float GfPlotDefs::xPlotSize() const 
{
	return(x_plot_size);
}

float GfPlotDefs::yPlotSize() const 
{
	return(y_plot_size);
}

int GfPlotDefs::curvesVisible() const 
{
	return(plot_curves);
}

int GfPlotDefs::commentsVisible() const 
{
	return(plot_comments);
}

int GfPlotDefs::axesVisible() const 
{
	return(plot_axes);
}

int GfPlotDefs::font() const 
{
	return(axes_font);
}

int GfPlotDefs::color() const 
{
	return(axes_color);
}

int GfPlotDefs::defaults()
{
	free(plot_name);
	init();
	return(GF_OK);
}



