#include "gfcommon.h"
#include <qfile.h>
#include "gfscale.h"

#ifndef GFPROFILEH
#define GFPROFILEH

/*

	GfProfile class 
	


*/

#define EOF_AUTO 0
#define EOF_YES 1
#define EOF_NO 2

#define FMT_FIXED 0
#define FMT_DELIM 1


class GfProfile
{

public:
	GfProfile(); 
	GfProfile(const GfProfile &); // copy constructor
	GfProfile(const GfProfile *); // copy constructor
	
	virtual ~GfProfile();


	int setEof(int );
	int setLineFormat(int );
	int setDelimChar(char );
	int setDataLength(int );
	int setInvecLength(int );
	
	int eof() const ;
	int lineFormat() const ;
	char delimChar() const ;
	int dataLength() const ;
	int invecLength() const ;
	
	void defaults();
	
	void print(OSTREAM *);
	int readFormat(QFile *in,char *line);
	
	void printScale(OSTREAM *os,GfScale *s);
	int readScale(QFile *in,GfScale *s);
	
	void operator=(const GfProfile &); // assignment operator
	

private:

	int use_eof;
	int use_delim;     /* use delimeter charater on reads 1==yes */
	char delim_char;    /* delimeter character */
	int dlen;         /* the data length and invec length formats */
	int vlen;


	void init();
	
	int parse_data_fixed(char *line,int *i1, float *x, 
		float *y,int xcol,int ycol,int vlen2);
	void strip_crlf(char *text);

	
};

OSTREAM &operator<<(OSTREAM &os, GfProfile *m);
OSTREAM &operator<<(OSTREAM &os, GfProfile &m);


#endif // GfProfile
